@@ -318,7 +318,8 @@ class Collection(collection.Collection):
             return result
         return None
 
-    def find_and_modify(self, query=None, update=None, fields=None, upsert=False, **kwargs):
+    def find_and_modify(self, query=None, update=None, fields=None,
+                        upsert=False, remove=False, **kwargs):
         if query is None: query = {}
         before = self.find_one(query, sort=kwargs.get('sort'))
         upserted = False
@@ -329,18 +330,21 @@ class Collection(collection.Collection):
             else:
                 return None
         before = self.find_one(query, fields, sort=kwargs.get('sort'))
-        if not upserted:
+        if remove:
+            self.remove({'_id': before['_id']})
+        elif not upserted:
             self.update({'_id': before['_id']}, update)
         if kwargs.get('new', False) or upserted:
             return self.find_one(dict(_id=before['_id']), fields)
 
         return before
 
-    def insert(self, doc_or_docs, safe=False):
+    def insert(self, doc_or_docs, manipulate=True, safe=False, **kwargs):
         if not isinstance(doc_or_docs, list):
             doc_or_docs = [ doc_or_docs ]
         for doc in doc_or_docs:
-            doc = bcopy(doc)
+            if not manipulate:
+                doc = bcopy(doc)
             bson_safe(doc)
             _id = doc.get('_id', ())
             if _id == ():
@@ -416,7 +420,7 @@ class Collection(collection.Collection):
         if not unique: return
         self._unique_indexes[keys] = index = {}
         for id, doc in self._data.iteritems():
-            key_values = tuple(doc.get(key[0], None) for key in keys)
+            key_values = self._extract_index_key(doc, keys)
             index[key_values] =id
         return index_name
 
@@ -438,10 +442,17 @@ class Collection(collection.Collection):
     def __repr__(self):
         return 'mim.Collection(%r, %s)' % (self._database, self.name)
 
+    def _extract_index_key(self, doc, keys):
+        key_values = list()
+        for key in keys:
+            sub, key = _traverse_doc(doc, key[0])
+            key_values.append(sub.get(key, None))
+        return tuple(key_values)
+
     def _index(self, doc):
         if '_id' not in doc: return
         for keys, index in self._unique_indexes.iteritems():
-            key_values = tuple(doc.get(key[0], None) for key in keys)
+            key_values = self._extract_index_key(doc, keys)
             old_id = index.get(key_values, ())
             if old_id == doc['_id']: continue
             if old_id in self._data:
@@ -450,7 +461,7 @@ class Collection(collection.Collection):
 
     def _deindex(self, doc):
         for keys, index in self._unique_indexes.iteritems():
-            key_values = tuple(doc.get(key[0], None) for key in keys)
+            key_values = self._extract_index_key(doc, keys)
             index.pop(key_values, None)
 
     def map_reduce(self, map, reduce, out, full_response=False, **kwargs):
@@ -528,7 +539,18 @@ class Cursor(object):
         # Le *sigh* -- this is the only place apparently where pymongo *does*
         # clone
         clone = self.clone()
-        return clone.skip(key).next()
+        if isinstance(key, slice):
+            _clone = clone
+            start, end = key.start, key.stop # step not supported
+            if start is None:
+                start = 0
+            _clone = _clone.skip(start)
+            if end is not None:
+                _clone = _clone.limit(end-start)
+            return _clone
+        elif isinstance(key, int):
+            return clone.skip(key).next()
+        raise TypeError('indicies must be integers, not %s' % type(key))
 
     def __iter__(self):
         return self
@@ -811,6 +833,8 @@ class MatchDoc(Match):
             return self, first
         if first not in self._doc:
             self._doc[first] = MatchDoc({})
+        if self._doc[first] is None:
+            return MatchDoc({}), None
         return self[first].traverse(*rest)
     def iteritems(self):
         return self._doc.iteritems()
@@ -831,6 +855,8 @@ class MatchDoc(Match):
     def setdefault(self, key, default):
         self._doc.setdefault(key, default)
         return self._orig.setdefault(key, default)
+    def keys(self):
+        return self._doc.keys()
 
 
 class MatchList(Match):
